/**
 * DevExtreme (esm/renovation/utils/plugin/context.js)
 * Version: 22.1.0 (build 22018-0314)
 * Build date: Tue Jan 18 2022
 *
 * Copyright (c) 2012 - 2022 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    createContext
} from "@devextreme/runtime/inferno";
var nextEntityId = 1;
export class PluginEntity {
    constructor() {
        this.id = nextEntityId;
        nextEntityId += 1
    }
    getValue(value) {
        return value
    }
}
export class PluginGetter extends PluginEntity {
    constructor(defaultValue) {
        super();
        this.defaultValue = defaultValue
    }
    getValue(value) {
        if (!value) {
            return this.defaultValue
        }
        return value.reduce((base, item) => item.func(base), this.defaultValue)
    }
}
export class PluginSelector extends PluginEntity {
    constructor(deps, func) {
        super();
        this.deps = deps;
        this.func = func
    }
}
export function createValue() {
    return new PluginEntity
}
export function createSelector(deps, func) {
    return new PluginSelector(deps, func)
}
export function createGetter(defaultValue) {
    return new PluginGetter(defaultValue)
}
export function createPlaceholder() {
    return new PluginEntity
}
export class Plugins {
    constructor() {
        this.items = {};
        this.subscriptions = {};
        this.subscribedSelectors = {}
    }
    set(entity, value) {
        var force = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : false;
        if (entity.id in this.items && this.items[entity.id] === value && !force) {
            return
        }
        this.items[entity.id] = value;
        var subscriptions = this.subscriptions[entity.id];
        if (subscriptions) {
            var callbackValue = entity.getValue(value);
            subscriptions.forEach(handler => {
                handler(callbackValue)
            })
        }
    }
    extend(entity, order, func) {
        var value = this.items[entity.id] || [];
        var insertIndex = value.filter(item => item.order < order).length;
        var item = {
            order: order,
            func: func
        };
        value.splice(insertIndex, 0, item);
        this.set(entity, value, true);
        return () => {
            var index = value.indexOf(item);
            if (index >= 0) {
                value.splice(index, 1);
                this.set(entity, value, true)
            }
        }
    }
    extendPlaceholder(entity, order, component) {
        var deps = arguments.length > 3 && void 0 !== arguments[3] ? arguments[3] : [];
        var value = this.items[entity.id] || [];
        var insertIndex = value.filter(item => item.order < order).length;
        var item = {
            order: order,
            component: component,
            deps: deps
        };
        value.splice(insertIndex, 0, item);
        this.set(entity, value, true);
        return () => {
            var index = value.indexOf(item);
            if (index >= 0) {
                value.splice(index, 1);
                this.set(entity, value, true)
            }
        }
    }
    getValue(entity) {
        this.update(entity);
        var value = this.items[entity.id];
        return entity.getValue(value)
    }
    hasValue(entity) {
        return entity.id in this.items
    }
    updateSelectorValue(entity) {
        var childValues = entity.deps.map(childEntity => this.getValue(childEntity));
        var newValue = entity.func.apply(null, childValues);
        this.set(entity, newValue)
    }
    subscribeToSelectorDeps(entity) {
        if (!this.subscribedSelectors[entity.id]) {
            this.subscribedSelectors[entity.id] = true;
            entity.deps.forEach(childEntity => {
                var childSubscriptions = this.getSubscriptions(childEntity);
                childSubscriptions.push(() => {
                    this.update(entity)
                })
            })
        }
    }
    update(entity) {
        if (entity instanceof PluginSelector) {
            entity.deps.forEach(child => {
                this.update(child)
            });
            this.subscribeToSelectorDeps(entity);
            if (entity.deps.every(childEntity => this.hasValue(childEntity))) {
                this.updateSelectorValue(entity)
            }
        }
    }
    getSubscriptions(entity) {
        var subscriptions = this.subscriptions[entity.id] || [];
        this.subscriptions[entity.id] = subscriptions;
        return subscriptions
    }
    watch(entity, callback) {
        this.update(entity);
        if (this.hasValue(entity)) {
            var value = this.items[entity.id];
            var callbackValue = entity.getValue(value);
            callback(callbackValue)
        }
        var subscriptions = this.getSubscriptions(entity);
        subscriptions.push(callback);
        return () => {
            var index = subscriptions.indexOf(callback);
            if (index >= 0) {
                subscriptions.splice(index, 1)
            }
        }
    }
    callAction(entity) {
        var value = this.getValue(entity);
        for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key]
        }
        return null === value || void 0 === value ? void 0 : value(...args)
    }
}
export var PluginsContext = createContext(null);
